package it.unibs.fp.patenteelibbrettopregoh;

import javax.xml.stream.*;
import java.io.*;
import java.util.ArrayList;

public class Parser {
    public static ArrayList<Persona> parsePersone(Formato formato, String fileName) {
        ArrayList<Persona> persone = new ArrayList<>();
        if (formato == Formato.XML) {
            fileName = fileName + ".xml";
            XMLInputFactory xmlif;
            XMLStreamReader xmlr;
            String currentElement = null;
            String nome = null;
            String cognome = null;
            String sesso = null;
            String dataNascita = null;
            String comuneNascita = null;
            String codiceFiscale = null;
            String scadenzaDocumento = null;

            try (FileInputStream reader = new FileInputStream(fileName)) {
                xmlif = XMLInputFactory.newInstance();
                xmlr = xmlif.createXMLStreamReader(fileName, reader);

                while (xmlr.hasNext()) {
                    int event = xmlr.getEventType();
                    switch (event) {
                        case XMLStreamConstants.START_ELEMENT:
                            currentElement = xmlr.getLocalName(); // <- sposta qui, nel posto giusto
                            break;
                        case XMLStreamConstants.CHARACTERS:
                            String text = xmlr.getText().trim();
                            if (!text.isEmpty() && currentElement != null) {
                                switch (currentElement) {
                                    case "nome" -> nome = text;
                                    case "cognome" -> cognome = text;
                                    case "sesso" -> sesso = text;
                                    case "data_nascita" -> dataNascita = text;
                                    case "comune_nascita" -> comuneNascita = text;
                                    case "codice_fiscale" -> codiceFiscale = text;
                                    case "scadenza_documento" -> scadenzaDocumento = text;
                                }
                            }
                            break;
                        case XMLStreamConstants.END_ELEMENT:
                            String endTag = xmlr.getLocalName();
                            if (endTag.equals("persona")) {
                                Persona persona = new Persona(nome, cognome, sesso, dataNascita, comuneNascita, codiceFiscale, scadenzaDocumento);
                                persone.add(persona);
                            }
                            currentElement = null; // reset solo alla fine di un elemento
                            break;
                    }
                    xmlr.next(); // importante: avanza solo una volta per ciclo
                }
                xmlr.close();
            } catch (FactoryConfigurationError | XMLStreamException | IOException e) {
                System.out.println("Error reading the file: " + e.getMessage());
            }

        }
        else {
            fileName = fileName + ".json";
        }
        return persone;
    }

    public static Comuni parseComuni(Formato formato, String fileName) {
        Comuni comuni = new Comuni();
        if (formato == Formato.XML) {
            fileName = fileName + ".xml";
            XMLInputFactory xmlif;
            XMLStreamReader xmlr;
            String currentElement = null;
            String nome = null;
            String codice = null;

            try (FileInputStream reader = new FileInputStream(fileName)) {
                xmlif = XMLInputFactory.newInstance();
                xmlr = xmlif.createXMLStreamReader(fileName, reader);

                while (xmlr.hasNext()) {
                    int event = xmlr.getEventType();

                    switch (event) {

                        case XMLStreamConstants.START_ELEMENT:
                            currentElement = xmlr.getLocalName(); // <- sposta qui, nel posto giusto
                            break;


                        case XMLStreamConstants.CHARACTERS:
                            String text = xmlr.getText().trim();
                            if (!text.isEmpty() && currentElement != null) {
                                if (currentElement.equals("nome")) {
                                    nome = text;
                                } else if (currentElement.equals("codice")) {
                                    codice = text;
                                }
                            }
                            break;

                        case XMLStreamConstants.END_ELEMENT:
                            String endTag = xmlr.getLocalName();
                            if (endTag.equals("comune")) {
                                Comune comune = new Comune(nome, codice);
                                comuni.addComune(comune);

                                nome = null;
                                codice = null;
                            }
                            currentElement = null; // reset solo alla fine di un elemento
                            break;
                    }

                    xmlr.next(); // importante: avanza solo una volta per ciclo
                }

                xmlr.close();
            } catch (FactoryConfigurationError | XMLStreamException | IOException e) {
                System.out.println("Error reading the file: " + e.getMessage());
            }
        }
        else {
            fileName = fileName + ".json";
        }
        return comuni;
    }

    public static void saveDiario (String fileName, ArrayList<PaginaDiario> pagineDiario, Formato formato) {
        // non so ancora come fare
        return;
    }
}
